#ifndef COVER_H
#define COVER_H

#include <vector>
#include <map>
#include <ext/hash_map>
#include <set>
#include "../geom/vec_2f.h"
#include "../geom/vec_2i.h"
#include "../geom/circle.h"
#include "../util/mathops.h"
#include "../gui/brush.h"
#include "coversection.h"
#include "coverscore.h"

#include <iostream>
using namespace std;


class Cover
{
  public:

    Cover ( const float scale, const float maxradius );
      // scale = distance between points
      // maxradius = maximum circle radius to accomodate efficiently

    ~Cover ();

    void cover ( const circle & area, const unsigned int id );
    void uncover ( const circle & area, const unsigned int id );
    void uncoverAll ();
      // un/cover the specified area with given id

    void setTarget ( const circle & area, const bool target );
    	// target/untarget area
    void unTargetAll ();

    float targetArea () const; // return raw area marked

    const CoverScore & getScore () const;

    void drawCells ( Brush & brush ) const;
    void drawPoints ( Brush & brush ) const;
    void drawSections ( Brush & brush ) const;
    void markCells ( Brush & brush, const circle & mark );

  private:

    CoverSection * getSection ( const vec_2i & coord );
      // get section by section grid coordinate

    CoverSection * getSection ( const vec_2f & point );
      // get section containing world coordinate

    std::set< CoverSection * > getSections ( const circle & area );
      // get all sections relevant to (overlapping) area

    void reinitScore ();
    	// reset score and reinitialize it from the sections

    //////////////////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////////////////

    CoverScore m_score;

    const float m_scale; // distance from each point to nearest neighbors
    const vec_2f m_gridorigin;
    const float m_sectionheight;
    const float m_sectionwidth;

    // TODO key type should be vec_2i, with proper operators defined there
    typedef __gnu_cxx::hash_map< unsigned int, CoverSection * > sectmap_t;
    sectmap_t m_sections;
};


#endif // COVER_H
