
#include "coverscore.h"


bool CoverScore::operator>= ( const CoverScore & rhs ) const
{
  std::vector< unsigned int > tncover = m_tcover.getTallies();
  std::vector< unsigned int > ntncover = m_ntcover.getTallies();
  std::vector< unsigned int > r_tncover = rhs.m_tcover.getTallies();
  std::vector< unsigned int > r_ntncover = rhs.m_ntcover.getTallies();

  unsigned int maxsize = tncover.size();
  maxsize = maxsize < ntncover.size() ? ntncover.size() : maxsize;
  maxsize = maxsize < r_tncover.size() ? r_tncover.size() : maxsize;
  maxsize = maxsize < r_ntncover.size() ? r_ntncover.size() : maxsize;
  tncover.resize( maxsize, 0 );
  ntncover.resize( maxsize, 0 );
  r_tncover.resize( maxsize, 0 );
  r_ntncover.resize( maxsize, 0 );


  // delta values to calculate
  float tscore = 0; // target cover score
  float ntscore = 0; // nontarget cover score
  int thits = 0; // target points with >0 coverage

  for( unsigned int i = 1; i < maxsize; i++ )
  {
    const float weight = (float)i * ( 1.0 - 0.05 * ( i - 1 ) );
    tscore += weight * ( (int)tncover[ i ] - (int)r_tncover[ i ] );
    ntscore += weight * ( (int)ntncover[ i ] - (int)r_ntncover[ i ] );
    thits += (int)tncover[ i ] - (int)r_tncover[ i ];
  }

  if( thits != 0 )
    return thits > 0;

  if( tscore != 0 )
    return tscore > 0;

  if( ntscore != 0 )
    return ntscore > 0;

  return true;
}


void CoverScore::applyPoint ( const coverpoint_t & point )
{
  if( point.target )
    m_tcover.incr( point.cover.members() );
  else
    m_ntcover.incr( point.cover.members() );
}


void CoverScore::unapplyPoint ( const coverpoint_t & point )
{
  if( point.target )
    m_tcover.decr( point.cover.members() );
  else
    m_ntcover.decr( point.cover.members() );
}


void CoverScore::clear ()
{
  m_tcover.clear();
  m_ntcover.clear();
}


const CoverScore & CoverScore::operator+= ( const CoverScore & rhs )
{
  m_tcover += rhs.m_tcover;
  m_ntcover += rhs.m_ntcover;
  return *this;
}


const CoverScore & CoverScore::operator-= ( const CoverScore & rhs )
{
  m_tcover -= rhs.m_tcover;
  m_ntcover -= rhs.m_ntcover;
  return *this;
}
