
#include "coversection.h"


CoverSection::CoverSection ( const rect_f & area, const float pointspacing, const vec_2f & gridorigin )
{
  m_bound = area;

  m_hexradius = 0.5 * pointspacing / SIN60;
  m_pointarea = 3 * SIN60 * m_hexradius * m_hexradius;

  // generate points
  //  all points are   bottom <= y < top, left <= x < right
  // TODO: a less stupid method
  const float rowspacing = pointspacing * SIN60;
  int row = (int)( ( area.bottom() - gridorigin.y ) / rowspacing );
  while( gridorigin.y + row * rowspacing > area.bottom() ) // double-checking...
    row--;
  while( gridorigin.y + row * rowspacing < area.bottom() )
    row++;

  const int mincol = (int)( ( area.left() - gridorigin.x ) / pointspacing ) - 2;

  m_score.clear();

  coverpoint_t p;
  p.cover.clear();
  p.target = false;
  while( gridorigin.y + row * rowspacing < m_bound.top() )
  {
    vec_2f roworigin( gridorigin + vec_2f( 0, row * rowspacing ) );
    if( row % 2 != 0 )
      roworigin += vec_2f( pointspacing * 0.5, 0 );
    int col = mincol;
    while( roworigin.x + col * pointspacing < m_bound.right() )
    {
      p.pos = roworigin + vec_2f( col * pointspacing, 0 );
      if( p.pos.x >= m_bound.left() )
        m_points.push_back( p );
      m_score.applyPoint( p );
      col++;
    }
    row++;
  }
}


CoverSection::~CoverSection ()
{
}


float CoverSection::coveredArea () const
{
  // TODO use the coverscores instead
  unsigned int tally = 0;
  for( unsigned pi = 0; pi < m_points.size(); pi++ )
    if( m_points[ pi ].cover.members() > 0 )
      tally++;
  return tally * m_pointarea;
}


void CoverSection::cover ( const circle & area, const unsigned int id )
{
  for( unsigned pi = 0; pi < m_points.size(); pi++ )
  {
  	coverpoint_t & p = m_points[ pi ];
    if( area.contains( p.pos ) )
    {
    	m_score.unapplyPoint( p );
      p.cover.insert( id );
      m_score.applyPoint( p );
    }
  }
}


void CoverSection::uncover ( const circle & area, const unsigned int id )
{
  for( unsigned pi = 0; pi < m_points.size(); pi++ )
  {
  	coverpoint_t & p = m_points[ pi ];
    if( area.contains( p.pos ) )
    {
    	m_score.unapplyPoint( p );
      p.cover.remove( id );
      m_score.applyPoint( p );
    }
  }
}


void CoverSection::uncoverAll ()
{
  for( unsigned int pi = 0; pi < m_points.size(); pi++ )
  {
    coverpoint_t & p = m_points[ pi ];
    m_score.unapplyPoint( p );
    p.cover.clear();
    m_score.applyPoint( p );
  }
}


void CoverSection::setTarget ( const circle & area, const bool target )
{
  for( unsigned pi = 0; pi < m_points.size(); pi++ )
  {
  	coverpoint_t & p = m_points[ pi ];
    if( area.contains( p.pos ) && p.target != target )
    {
    	m_score.unapplyPoint( p );
      p.target = target;
      m_score.applyPoint( p );
    }
  }
}


void CoverSection::unTargetAll ()
{
  for( unsigned pi = 0; pi < m_points.size(); pi++ )
  {
  	coverpoint_t & p = m_points[ pi ];
    if( p.target )
    {
    	m_score.unapplyPoint( p );
      p.target = false;
      m_score.applyPoint( p );
    }
  }
}


float CoverSection::targetArea () const
{
  unsigned int count = 0;
  for( unsigned pi = 0; pi < m_points.size(); pi++ )
    if( m_points[ pi ].target )
      count++;
  return (float)count * m_pointarea;
}


vec_2f CoverSection::closestPoint ( const vec_2f & point ) const
{
  if( m_points.empty() )
    return point;
  vec_2f closest = m_points[ 0 ].pos;
  float closestd = ( closest - point ).magsqr();
  for( unsigned int i = 1; i < m_points.size(); i++ )
  {
    float d = ( m_points[ i ].pos - point ).magsqr();
    if( d < closestd )
    {
      closest = m_points[ i ].pos;
      closestd = d;
    }
  }
  return closest;
}


void CoverSection::drawPoints ( Brush & brush ) const
{
  for( unsigned int pi = 0; pi < m_points.size(); pi++ )
  {
    if( m_points[ pi ].cover.members() > 0 )
      brush.setRGBA( 0.0, 0.0, 0.0, 0.8 );
    else
      brush.setRGBA( 0.0, 0.0, 0.0, 0.1 );
    brush.drawX( m_points[ pi ].pos, 0.1 );
  }
}


void CoverSection::drawCells ( Brush & brush ) const
{
  brush.setRGBA( 0.1, 0.7, 0.1, 1.0 );
  MWCRand rnd;
  rnd.seed( (unsigned int)this );
  for( unsigned int pi = 0; pi < m_points.size(); pi++ )
  {
  	const coverpoint_t & p = m_points[ pi ];
  	float r = rnd.rand();
    if( p.cover.members() > 0 )
    {
      brush.setRGBA( 0.5, 0.8 + 0.1 * r, 0.5, 1.0 );
      brush.fillHex( p.pos, m_hexradius );
    }
    if( p.target )
    {
      brush.setRGBA( 1.0, 0.1, 0.1, 0.4 );
      brush.fillHex( p.pos, m_hexradius );
    }
  }
}


void CoverSection::markCells ( Brush & brush, const circle & area ) const
{
  brush.setRGBA( 0.2, 0.2, 0.9, 0.3 );
  for( unsigned int pi = 0; pi < m_points.size(); pi++ )
  {
    if( area.contains( m_points[ pi ].pos ) )
      brush.fillHex( m_points[ pi ].pos, m_hexradius );
  }
}
