#ifndef COVERSECTION_H
#define COVERSECTION_H

#include <vector>
#include "../geom/vec_2f.h"
#include "../geom/circle.h"
#include "../gui/brush.h"
#include "../util/mathconsts.h"
#include "../util/mwcrand.h"
#include "../util/quickset.h"
#include "coverscore.h"
#include "coverpoint.h"


class CoverSection
{
  public:
    CoverSection ( const rect_f & area, const float pointspacing, const vec_2f & gridorigin = vec_2f(0,0) );
      // pointspacing is the distance from each point to its nearest neighbors
      // gridorigin is the point to base the grid upon

    ~CoverSection ();

    inline const rect_f & getBound () const
      { return m_bound; }

    inline const CoverScore & getScore () const
      { return m_score; }

    float coveredArea () const;
      // area having >0 coverage, in units squared

    void cover ( const circle & area, const unsigned int id );
    void uncover ( const circle & area, const unsigned int id );
    void uncoverAll ();
      // un/cover the specified area with given id

    void setTarget ( const circle & area, const bool target );
      // flag an area as targetted/not targetted
    void unTargetAll ();

    float targetArea () const;

    vec_2f closestPoint ( const vec_2f & point ) const;
      // return grid point closest to given point

    void drawPoints ( Brush & brush ) const;
    void drawCells ( Brush & brush ) const;
    void markCells ( Brush & brush, const circle & area ) const;


  private:

    rect_f m_bound; // area this section represents

    std::vector< coverpoint_t > m_points;
    float m_hexradius; // radius of hexagon each point represents
    float m_pointarea; // area each point represents

    CoverScore m_score;
};


#endif // COVERSECTION_H
