#ifndef CIRCLE_H
#define CIRCLE_H

#include "vec_2f.h"
#include "rect_f.h"
#include "../util/mathconsts.h"
#include <cmath>


class circle
{
  public:
    circle ( const vec_2f & center, const float radius )
      : m_center( center ), m_radius( fabsf( radius ) ),
        m_radiussqr( m_radius * m_radius ) {}

    inline const vec_2f & center () const
      { return m_center; }

    inline const float & radius () const
      { return m_radius; }

    inline float area () const
      { return TWOPI * 0.5 * m_radius * m_radius; }

    inline bool contains ( const vec_2f & point ) const
     { return ( m_center - point ).magsqr() <= m_radiussqr; }

    inline bool intersects ( const rect_f & rect ) const
    {
      if( m_center.x >= rect.right() + m_radius ||
          m_center.x < rect.left() - m_radius ||
          m_center.y >= rect.top() + m_radius ||
          m_center.y < rect.bottom() - m_radius )
        return false;

      if( m_center.x >= rect.right() )
      {
        if( m_center.y >= rect.top() )
          if( !contains( rect.corner_tr() ) )
            return false;
        if( m_center.y < rect.bottom() )
          if( !contains( rect.corner_br() ) )
            return false;
      }
      else if( m_center.x < rect.left() )
      {
        if( m_center.y >= rect.top() )
          if( !contains( rect.corner_tl() ) )
            return false;
        if( m_center.y < rect.bottom() )
          if( !contains( rect.corner_bl() ) )
            return false;
      }

      return true;
    }

  private:
    const vec_2f m_center;
    const float m_radius;
    const float m_radiussqr; // R^2, for contains()
};

#endif // CIRCLE_H
