
#ifndef RECT_H
#define RECT_H

#include "vec_2f.h"

class rect_f
{
  public:
    rect_f ()
      : m_l( 0.0 ), m_r( 0.0 ), m_t( 0.0 ), m_b( 0.0 ) {}

    rect_f ( const float left, const float right, const float top, const float bottom )
    	: m_l( left ), m_r( right ), m_t( top ), m_b( bottom ) {}

    rect_f ( const vec_2f & tl, const vec_2f & br )
    	: m_l( tl.x ), m_r( br.x ), m_t( tl.y ), m_b( br.y ) {}

    inline const float & left () const { return m_l; }
    inline const float & right () const { return m_r; }
    inline const float & top () const { return m_t; }
    inline const float & bottom () const { return m_b; }

    inline vec_2f corner_tl () const { return vec_2f( m_l, m_t ); }
    inline vec_2f corner_tr () const { return vec_2f( m_r, m_t ); }
    inline vec_2f corner_bl () const { return vec_2f( m_l, m_b ); }
    inline vec_2f corner_br () const { return vec_2f( m_r, m_b ); }

    inline float height () const { return m_t - m_b; }
    inline float width () const { return m_r - m_l; }

    inline vec_2f center () const
    	{	return vec_2f( ( m_r + m_l ) * 0.5, ( m_t + m_b ) * 0.5 ); }

    inline rect_f scale ( const float dif ) const
    {
    	return rect_f(  center().x - width() * 0.5 - dif,
                      center().x + width() * 0.5 + dif,
                      center().y + height() * 0.5 + dif,
                      center().y - height() * 0.5 - dif );
    }

    inline rect_f expand ( const float dist ) const
      // produce a rect with the same center with +2*dist height and width
      { return rect_f( m_l - dist, m_r + dist, m_t + dist, m_b - dist ); }

    inline rect_f expand ( const vec_2f point ) const
    {
      rect_f exp( *this );
      if( point.x < exp.m_l )
        exp.m_l = point.x;
      if( point.x > exp.m_r )
        exp.m_r = point.x;
      if( point.y < exp.m_b )
        exp.m_b = point.y;
      if( point.y > exp.m_t )
        exp.m_t = point.y;
      return exp;
    }

    inline bool contains ( const vec_2f & point ) const
      { return point.x >= m_l && point.x < m_r && point.y >= m_b && point.y < m_t; }

  private:
    float m_l, m_r, m_t, m_b;
};

#endif
