
#ifndef VEC_2F_H
#define VEC_2F_H

#include <cmath>


class vec_2f
{
	public:
		float x, y;

		vec_2f () : x( 0 ), y( 0 ) {}
		vec_2f ( const float newx, const float newy ) : x( newx ), y( newy ) {}

	  ///////////////////////////////////////////////
	  /////////////////////////////////////// set ops

	  inline void setzero ()
      { x = y = 0.0; }

	  inline void setnormal ( const float angle )
	  {
	    x = cosf( angle );
	    y = sinf( angle );
	  }

    inline void floor ()
	  {
	  	x = floorf( x );
	  	y = floorf( y );
	  }

	  //////////////////////////////////////////////////
	  /////////////////////////////////////// vector ops

	  inline vec_2f operator+ ( const vec_2f & rhs ) const
	  {
	    vec_2f v;
	    v.x = x + rhs.x;
	    v.y = y + rhs.y;
	    return v;
	  }

	  inline vec_2f operator- ( const vec_2f & rhs ) const
	  {
	    vec_2f v;
	    v.x = x - rhs.x;
	    v.y = y - rhs.y;
	    return v;
	  }

	  inline vec_2f operator- () const
	  {
	  	vec_2f v;
	  	v.x = -x;
	  	v.y = -y;
	  	return v;
	  }

	  inline vec_2f & operator+= ( const vec_2f & rhs )
	  {
	    x += rhs.x;
	    y += rhs.y;
	    return *this;
	  }

	  inline vec_2f & operator-= ( const vec_2f & rhs )
	  {
	    x -= rhs.x;
	    y -= rhs.y;
	    return *this;
	  }

	  inline vec_2f normal () const
	  {
	    const float mag = magnitude();
	    if( mag == 0 )
	    	return *this;
	    return (*this) / mag;
	  }

	  inline vec_2f rotate ( const float radians ) const
	  {
	    const float sintheta = sinf( radians );
	    const float costheta = cosf( radians );
	    vec_2f v;
	    v.x = costheta * x - sintheta * y;
	    v.y = sintheta * x + costheta * y;
	    return v;
	  }

	  //////////////////////////////////////////////////
	  /////////////////////////////////////// scalar ops

	  inline vec_2f operator* ( const float scalar ) const
	  {
	    vec_2f v;
	    v.x = x * scalar;
	    v.y = y * scalar;
	    return v;
	  }

	  inline vec_2f operator/ ( const float scalar ) const
	  {
	    vec_2f v;
	    v.x = x / scalar;
	    v.y = y / scalar;
	    return v;
	  }

	  inline vec_2f & operator*= ( const float scalar )
	  {
	    x *= scalar;
	    y *= scalar;
	    return *this;
	  }

	  inline vec_2f & operator/= ( const float scalar )
	  {
	    x /= scalar;
	    y /= scalar;
	    return *this;
	  }

	  inline float magsqr () const
      { return x * x + y * y; }

	  inline float magnitude () const
      { return sqrtf( x * x + y * y ); }

    // determine angle from this vector to next
    inline float angle ( const vec_2f & next ) const
    {
      if( this->magsqr() == 0.0 || next.magsqr() == 0.0 )
        return 0.0;
      return 0.0;
    }

};

#endif
