
#ifndef VEC_2I_H
#define VEC_2I_H

#include <cmath>
#include <vector>

class vec_2i
{
	public:
		int x, y;

		vec_2i () : x( 0 ), y( 0 ) {}
		vec_2i ( const int newx, const int newy ) : x( newx ), y( newy ) {}
		vec_2i ( const vec_2i & src ) : x( src.x ), y( src.y ) {}

	  ///////////////////////////////////////////////
	  /////////////////////////////////////// set ops

	  inline vec_2i & operator= ( const vec_2i & rhs )
	  {
	    x = rhs.x;
	    y = rhs.y;
	    return *this;
	  }

	  inline void setzero ()
	  {
	    x = y = 0;
	  }

	  ///////////////////////////////////////////////////
	  /////////////////////////////////////// compare ops

	  inline bool operator== ( const vec_2i & rhs ) const
	  {
	  	return x == rhs.x && y == rhs.y;
	  }

	  inline bool operator!= ( const vec_2i & rhs ) const
	  {
	  	return x != rhs.x || y != rhs.y;
	  }

	  //////////////////////////////////////////////////
	  /////////////////////////////////////// vector ops

	  inline vec_2i operator+ ( const vec_2i & rhs ) const
	  {
	    vec_2i v;
	    v.x = x + rhs.x;
	    v.y = y + rhs.y;
	    return v;
	  }

	  inline vec_2i operator- ( const vec_2i & rhs ) const
	  {
	    vec_2i v;
	    v.x = x - rhs.x;
	    v.y = y - rhs.y;
	    return v;
	  }

	  inline vec_2i operator- () const
	  {
	  	vec_2i v;
	  	v.x = -x;
	  	v.y = -y;
	  	return v;
	  }

	  inline vec_2i & operator+= ( const vec_2i & rhs )
	  {
	    x += rhs.x;
	    y += rhs.y;
	    return *this;
	  }

	  inline vec_2i & operator+= ( const std::vector< vec_2i > & terms )
	  {
	  	for( unsigned int i = 0; i < terms.size(); i++ )
	  	{
	  		x += terms[ i ].x;
	  		y += terms[ i ].y;
	  	}
	  	return *this;
	  }

	  inline vec_2i & operator-= ( const vec_2i & rhs )
	  {
	    x -= rhs.x;
	    y -= rhs.y;
	    return *this;
	  }

	  //////////////////////////////////////////////////
	  /////////////////////////////////////// scalar ops

	  inline float magnitude () const
	  {
	  	return sqrtf( (float)( x * x + y * y ) );
	  }

	  inline unsigned int magSquared () const
	  {
	  	return (unsigned int)( x * x ) + (unsigned int)( y * y );
	  }

	  inline vec_2i operator* ( const int scalar ) const
	  {
	    vec_2i v;
	    v.x = x * scalar;
	    v.y = y * scalar;
	    return v;
	  }

	  inline vec_2i operator/ ( const int scalar ) const
	  {
	    vec_2i v;
	    v.x = x / scalar;
	    v.y = y / scalar;
	    return v;
	  }

	  inline vec_2i & operator*= ( const int scalar )
	  {
	    x *= scalar;
	    y *= scalar;
	    return *this;
	  }

	  inline vec_2i & operator/= ( const int scalar )
	  {
	    x /= scalar;
	    y /= scalar;
	    return *this;
	  }

};

#endif
