
#include "brush.h"

Brush::Brush ( SDLGLview & canvas )
	: m_canvas( canvas ), m_r( 1 ), m_g( 1 ), m_b( 1 ), m_a( 1 ),
    m_dl_hexagon( 0 ), m_dl_box( 0 )
{}

Brush::~Brush () {}


void Brush::setRGBA ( const float r, const float g, const float b, const float a )
{
	m_r = r;
	m_g = g;
	m_b = b;
	m_a = a;
}


void Brush::setA ( const float a )
{
  m_a = a;
}


void setHSVA ( const float h, const float s, const float v, const float a )
{

}


void Brush::drawSpot ( const vec_2f & center, const float radius ) const
{
	const float segments = 7;
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glLoadIdentity();
	glTranslatef( center.x, center.y, 0 );
	glPointSize( 1.0 );
	glLineWidth( 1.0 );
	glColor4f( m_r, m_g, m_b, m_a );
	glBegin( GL_POLYGON );
		for( float ang = 0; ang < TWOPI; ang += TWOPI / segments )
			glVertex3f( radius * cos(ang), radius * sin(ang), 0 );
	glEnd();
	glColor4f( 0, 0, 0, m_a );
	glBegin( GL_LINE_LOOP );
		for( float ang = 0; ang < TWOPI; ang += TWOPI / segments )
			glVertex3f( radius * cos(ang), radius * sin(ang), 0 );
	glEnd();
	glPopMatrix();
}


void Brush::drawX ( const vec_2f & center, const float size ) const
{
	glPointSize( 1.0 );
	glLineWidth( 1.0 );

  const float halfsize = size * 0.5;
  glBegin( GL_LINES );
    glVertex3f( center.x - halfsize, center.y - halfsize, 0 );
    glVertex3f( center.x + halfsize, center.y + halfsize, 0 );
    glVertex3f( center.x + halfsize, center.y - halfsize, 0 );
    glVertex3f( center.x - halfsize, center.y + halfsize, 0 );
  glEnd();
}


void Brush::drawLineStrip ( const std::vector< vec_2f > & points ) const
{
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glLoadIdentity();
	glPointSize( 1.0 );
	glLineWidth( 1.0 );
	glColor4f( m_r, m_g, m_b, m_a );
	glBegin( GL_LINE_STRIP );
		for( unsigned int i = 0; i < points.size(); i++ )
			glVertex3f( points[i].x, points[i].y, 0 );
	glEnd();
	glPopMatrix();
}


void Brush::drawBox ( const rect_f & bound )
{
  if( m_dl_box == 0 )
  {
    m_dl_box = glGenLists( 1 );
    glNewList( m_dl_box, GL_COMPILE );
      glBegin( GL_LINE_LOOP );
        glVertex3f( 0, 0, 0 );
        glVertex3f( 0, 1, 0 );
        glVertex3f( 1, 1, 0 );
        glVertex3f( 1, 0, 0 );
      glEnd();
    glEndList();

  }
	glMatrixMode( GL_MODELVIEW );
  glPushMatrix();
  glLoadIdentity();
	glTranslatef( bound.left(), bound.bottom(), 0 );
	glScalef( bound.width(), bound.height(), 1.0 );
	glPointSize( 1.0 );
	glLineWidth( 1.0 );
	glColor4f( m_r, m_g, m_b, m_a );
	glCallList( m_dl_box );
	glPopMatrix();
}


void Brush::fillBox ( const rect_f & bound ) const
{
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glLoadIdentity();
	glPointSize( 1.0 );
	glLineWidth( 1.0 );
	glColor4f( m_r, m_g, m_b, m_a );
	glBegin( GL_TRIANGLES );
		glVertex3f( bound.left(), bound.top(), 0 );
		glVertex3f( bound.right(), bound.top(), 0 );
		glVertex3f( bound.right(), bound.bottom(), 0 );

		glVertex3f( bound.left(), bound.top(), 0 );
		glVertex3f( bound.right(), bound.bottom(), 0 );
		glVertex3f( bound.left(), bound.bottom(), 0 );
	glEnd();
	glPopMatrix();
}


void Brush::fillHex ( const vec_2f & center, const float radius )
{
  if( m_dl_hexagon == 0 )
  {
    std::vector< vec_2f > unitverts;
    vec_2f v;
    for( float a = 0.0; a < TWOPI; a += TWOPI / 6.0 )
    {
      v.setnormal( a + TWOPI / 12 );
      unitverts.push_back( v );
    }

    m_dl_hexagon = glGenLists( 1 );
    glNewList( m_dl_hexagon, GL_COMPILE );
      glBegin( GL_POLYGON );
        for( unsigned int i = 0; i < 6; i++ )
          glVertex3f( unitverts[i].x, unitverts[i].y, 0 );
      glEnd();
    glEndList();
  }

  glPushMatrix();
	glLoadIdentity();
	glTranslatef( center.x, center.y, 0 );
	glScalef( radius, radius, 1.0 );
	glColor4f( m_r, m_g, m_b, m_a );
	glCallList( m_dl_hexagon );
	glPopMatrix();
}


void Brush::drawCircle ( const circle & circle ) const
{
  const float segments = 51;
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glLoadIdentity();
	glTranslatef( circle.center().x, circle.center().y, 0 );
	glColor4f( m_r, m_g, m_b, m_a );
	glBegin( GL_LINE_LOOP );
		for( float ang = 0; ang < TWOPI; ang += TWOPI / segments )
			glVertex3f( circle.radius() * cos(ang), circle.radius() * sin(ang), 0 );
	glEnd();
	glPopMatrix();
}


void Brush::drawBarGraph ( const barstyle_t & style, const std::vector< float > & data )
{
	if( data.size() < 2 )
		return;
	const float height = style.sizepos.height();
	const float spacing = 0.4 * style.barwidth;

	//setRGBA( 1.0, 1.0, 1.0, 0.8 );
	//fillBox( bound.scale( maxbar.height() * 0.05 ) );

	vec_2f pen( style.sizepos.corner_bl() );
	pen.x += spacing * 0.5;
	float a = 0.9;
	for( unsigned int i = 1; i < data.size(); i++ )
	{
		rect_f bar( pen + vec_2f( 0, -height * ( data[ i ] / data[ 0 ] - 1.0 ) ),
                pen + vec_2f( style.barwidth, height ) );
		a *= 0.9;
		a = a < 0.2 ? 0.2 : a;

    setRGBA( style.r, style.g, style.b, 1.0 );
		fillBox( bar );

		if( i == style.hilight )
		{
			setRGBA( 0.0, 0.8, 0.0, 0.7 );
			fillBox( bar );
		}

		pen.x += style.barwidth + spacing;
	}

  if( style.bound )
  {
    setRGBA( 0.0, 0.0, 0.0, 1.0 );
    drawBox( style.sizepos.expand( pen + vec_2f( spacing * 0.5, 0 ) ) );
  }
}


void Brush::print ( const vec_2f & pos, const std::string & text ) const
{

}
