#include "fpstimer.h"


FPSTimer::FPSTimer ( Uint32 fps )
{
  // set up timer
	fps_sem = SDL_CreateSemaphore( 0 );
	Uint32 interval = 1000 / fps;
	fps_timer = SDL_AddTimer( interval, &FPSTimer::WaitUnlock, fps_sem );

  lastticks = SDL_GetTicks();
  fpsticks = 0;
  totalticks = 0;
}


FPSTimer::~FPSTimer ()
{
	SDL_RemoveTimer( fps_timer );
	SDL_DestroySemaphore( fps_sem );
}


Uint32 FPSTimer::getTimestep ()
{
	Uint32 ticksnow = SDL_GetTicks();
	Uint32 step;
	if( ticksnow < lastticks ) // value has wrapped around
		step = (0xffffffff - lastticks) + ticksnow;
	else
		step = ticksnow - lastticks;

	lastticks = ticksnow;
	return step;
}


bool FPSTimer::probeTimer () const
{
	return SDL_SemValue( fps_sem ) > 0;
}


bool FPSTimer::probeTicks () const
{
	return fpsticks > 0;
}


unsigned int FPSTimer::getTicks ()
{
  if( fpsticks == 0 )
    return 0;
  unsigned int ticks = fpsticks;
  fpsticks = 0;
  return ticks;
}


unsigned long FPSTimer::totalTicks () const
{
  return totalticks;
}


void FPSTimer::handleEvent ( const SDL_Event & event )
{
  if( SDL_SemTryWait( fps_sem ) == 0 )
  {
    fpsticks++;
    totalticks++;
  }
}


Uint32 FPSTimer::WaitUnlock( Uint32 interval, void * semaphore )
{
  if( SDL_SemValue( (SDL_sem*)semaphore ) == 0 ){
    SDL_SemPost( (SDL_sem*)semaphore );
    SDL_Event evnt;
    evnt.type = SDL_USEREVENT;
    SDL_PushEvent( &evnt ); // wake up event loop
  }
  return interval;
}
