#ifndef FPSTIMER_H_
#define FPSTIMER_H_

#include <SDL/SDL.h>

/*
 * This class generates a SDL_USEREVENT on an interval.
 * The SDL timer subsystem must be initialized prior to instantiation.
 * any SDL_USEREVENT must be forwarded to this class
 */

class FPSTimer
{
	public:
		FPSTimer ( Uint32 fps );
		~FPSTimer ();

		Uint32 getTimestep ();
      // returns number of ms since last call

    bool probeTimer () const;
      // return true if a timer event is waiting to be handled

    bool probeTicks () const;
      // return true if getTicks will return >0, do not alter state

		unsigned int getTicks ();
      // get number of ticks ( frame intervals ) since last call

    unsigned long totalTicks () const;
      // get 'em all

		void handleEvent ( const SDL_Event & event );
      // forward any SDL_USEREVENT here

    ///////////////////////////////////////////////////////////////

		static Uint32 WaitUnlock ( Uint32 interval, void * semaphore );
      // callback for the SDL timer.
      // NOT FOR PUBLIC USE.
      // TODO figure out how to make this private

	private:
		Uint32 lastticks; // for getTimestep
		unsigned int fpsticks; // for getTicks
		unsigned long totalticks;
		SDL_sem * fps_sem;
		SDL_TimerID fps_timer;
};

#endif /*FPSTIMER_H_*/
