#ifndef SDLMOUSE_H_
#define SDLMOUSE_H_

#include <SDL/SDL.h>
#include <queue>
#include "../geom/vec_2f.h"

class SDLMouse
{
	public:
		SDLMouse ();

		void updateState ( const unsigned int timestamp = 0 );

		struct click_t // not just for clicks, but for any button state changes
		{
		  vec_2f point;
		  Uint8 buttons;
		  Uint8 buttonchanges; // 1 if the button has changed
		  unsigned int timestamp;
		};

    bool popClick ( click_t & click );
      // get the next queued click event
      // return true iff there actually is one

    void clearEvents ();
      // clear all queued events

		const vec_2f & getPoint () const;
      // get current pointer location

		vec_2f getMotion () const;
      // get movement since previous update

		bool buttonDown ( const Uint8 button ) const;
      // true if button is currently held down

	private:

		struct mstate
		{
			vec_2f point;
			Uint8 buttons;
		};
		mstate s_cur; // current state
		mstate s_prev; // previous state
		std::queue< click_t > m_clickq;
};

#endif /*SDLMOUSE_H_*/
