#ifndef VIEW_H_
#define VIEW_H_

#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>
#include "../geom/vec_2f.h"
#include "../geom/rect_f.h"

class SDLGLview
{
	public:
		SDLGLview ( int w, int h );
		~SDLGLview ();

		SDL_Surface * getScreen ();
		void resizeScreen ( int w, int h );
		void resizeScreen ( const SDL_ResizeEvent & resizeevent );

		void setCam ( const vec_2f & focus, float scale );
		void setCam ( const rect_f & area, float minscale );

		void setDragBase ( const vec_2f & viewbase );
      // viewbase is view coordinate base position
		void dragCam ( const vec_2f & topoint, float rate );
      // move the focus, in view coordinates
      // first call setDragBase, then call dragCam to update
      //  the pointer position from the original viewbase

		void panCam ( const vec_2f & target, float rate );
		float getTargetScale () const; // get the last scale setting
		void panScale ( float scale, float rate );
		void panTick ();
      // set pan target and advance towards, respectively
      // rate is [0,1], 1 meaning instant

    float getScale () const; // get current scale

		void fill ( float r, float g, float b, float a );
      // fill entire view

		vec_2f ViewToWorld ( const vec_2f view_point ) const;
      // return vector converted from view coords to world coords

	private:

		void updateProjection ();
      // update projection matrix to match new focus

		SDL_Surface * screen;
		int viewport_w, viewport_h;

		vec_2f cur_focus;
		float cur_scale;

		vec_2f target_focus;
		float target_scale;
		float pan_rate;

		vec_2f drag_base_view;
		vec_2f drag_base_world;
};

#endif /*VIEW_H_*/
