
#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>
#include "gui/fpstimer.h"
#include "util/mtrand.h"
#include "gui/view.h"
#include "gui/brush.h"
#include "gui/sdlmouse.h"
//#include "gui/font.h"
#include "util/mathops.h"
#include "simulation.h"


#include <iostream>
using namespace std;

// globals
MTRand mt;
MWCRand mwc;

int main (int argc, char ** argv)
{
  try{
  
  if( SDL_Init( SDL_INIT_TIMER | SDL_INIT_VIDEO ) < 0 )
    throw( "failed to init SDL" );
  atexit( SDL_Quit );
  SDLGLview view( 800, 600 );
  SDL_WM_SetCaption( "sweep", NULL );

  mt.seed();
  mwc.seed( mt.randInt() );

  FPSTimer fpstmr( 25 );
  Brush brush( view );
  //Font font( "/usr/share/fonts/truetype/freefont/FreeSansBold.ttf" );

  const Sensor basesensor( vec_2f( 0, 0 ), 200.0, 10.0, 0 );

  Simulation sim( basesensor.getRadius() );
  sim.setN( 0 );

  Simulation::drawopts_t drawopts;
  drawopts.cover = true;
  drawopts.sensors = true;
  drawopts.paths = true;

  bool showgraph = true;
  bool animate = false;

  // input
	SDLMouse mouse;
	Uint8 * keystate = NULL;

  view.setCam( vec_2f( 0, 0 ), basesensor.getRadius() / 5.0 );
  view.fill( 0.0, 0.0, 0.0, 1.0 );
  SDL_GL_SwapBuffers();

  bool done = false;
	while( !done )
  {
  	keystate = SDL_GetKeyState( NULL );

  	// handle events
    SDL_Event event;
    while( SDL_PollEvent( &event ) )
      switch( event.type )
      {
        case SDL_QUIT:
          done = true;
          break;

        case SDL_VIDEORESIZE:
          view.resizeScreen( event.resize );
          break;

        case SDL_KEYDOWN:
          switch( event.key.keysym.sym )
          {
            case SDLK_z:
            	view.panScale( view.getTargetScale() / 1.2 , 0.5 );
            	break;

            case SDLK_a:
            	view.panScale( view.getTargetScale() * 1.2 , 0.5 );
            	break;

             case SDLK_0:
            	sim.setN( 0 );
            	break;
            case SDLK_1:
            	sim.setN( 1 );
            	break;
            case SDLK_2:
            	sim.setN( 2 );
            	break;
            case SDLK_3:
            	sim.setN( 3 );
            	break;
            case SDLK_4:
            	sim.setN( 4 );
            	break;
            case SDLK_5:
            	sim.setN( 5 );
            	break;
            case SDLK_6:
            	sim.setN( 6 );
            	break;
            case SDLK_7:
            	sim.setN( 7 );
            	break;
            case SDLK_8:
            	sim.setN( 8 );
            	break;
            case SDLK_9:
            	sim.setN( 9 );
            	break;

            case SDLK_F1:
            	drawopts.sensors = !drawopts.sensors;
            	break;
            case SDLK_F2:
            	drawopts.paths = !drawopts.paths;
            	break;
            case SDLK_F3:
             	animate = !animate;
              if( !animate )
              	sim.resetTime();
             	break;
            case SDLK_F4:
             	drawopts.cover = !drawopts.cover;
            	break;

            case SDLK_F6:
            	showgraph = !showgraph;
            	break;

            case SDLK_BACKSPACE:
            	sim.scramble();
            	break;

            case SDLK_ESCAPE:
            	sim.clearSensors();
            	sim.clearTarget();
            	break;

            case SDLK_RETURN:
            	//sim.setTarget( rect_f( -14, 15, 15, -14 ) );
            	sim.addUniform( basesensor, rect_f( -14, 15, 15, -14 ), 100 );
            	break;

            case SDLK_RIGHT:
            	sim.optimizeN();
      	  		break;

            default:
            	break;
          }
          break;

        case SDL_MOUSEMOTION:
        case SDL_MOUSEBUTTONDOWN:
        case SDL_MOUSEBUTTONUP:
          mouse.updateState( SDL_GetTicks() );
          break;

        case SDL_USEREVENT:
          fpstmr.handleEvent( event );
        	break;

        default:
        	break;
      }


    if( fpstmr.probeTicks() )
    {
      fpstmr.getTicks();

      SDLMouse::click_t click;
      while( mouse.popClick( click ) )
      {
        if( click.buttonchanges & SDL_BUTTON( SDL_BUTTON_RIGHT ) )
          if( click.buttons & SDL_BUTTON( SDL_BUTTON_RIGHT ) )
            view.setDragBase( click.point );
          else
            view.dragCam( click.point, 0.5 );
        if( click.buttonchanges & click.buttons & SDL_BUTTON( SDL_BUTTON_LEFT ) && !keystate[ SDLK_LSHIFT ] )
          sim.addCluster( basesensor, view.ViewToWorld( click.point ), 0, 1 );
      }
      if( mouse.buttonDown( SDL_BUTTON_RIGHT ) )
        view.dragCam( mouse.getPoint(), 0.5 );


      if( mouse.buttonDown( SDL_BUTTON_LEFT ) && keystate[ SDLK_LSHIFT ] )
        sim.setTarget( circle( view.ViewToWorld( mouse.getPoint() ), basesensor.getRadius() * 1.0 ) );


	  	if( keystate[ SDLK_SPACE ] )
	  	{
	  		while( !fpstmr.probeTimer() )
	  			sim.optimizeN();
	  	}


      if( animate )
        sim.step( 0.01 );
      view.panTick();

      view.fill( 1.0, 1.0, 1.0, 1.0 );

	  	glMatrixMode( GL_MODELVIEW );

	  	sim.draw( brush, drawopts );

	  	if( showgraph )
	  	{
	  		Brush::barstyle_t graphstyle;
	  		const vec_2f origin = view.ViewToWorld( vec_2f( 10.0, 10.0 + 100.0 ) );
        graphstyle.sizepos = rect_f( origin, origin + vec_2f( 0, -100.0 / view.getScale() ) );
	  		graphstyle.barwidth = 15.0 / view.getScale();
	  		graphstyle.hilight = sim.N();
	  		graphstyle.bound = false;

	  		graphstyle.r = 0.6;
	  		graphstyle.g = 0.6;
	  		graphstyle.b = 0.9;
	  		brush.drawBarGraph( graphstyle, sim.getMaxNCover() );

	  		graphstyle.r = 0.1;
	  		graphstyle.g = 0.1;
	  		graphstyle.b = 0.1;
	  		brush.drawBarGraph( graphstyle, sim.getTargetNCover() );
	  	}

	  	SDL_GL_SwapBuffers();
    }

  	SDL_WaitEvent( NULL );
  } // !done

  } catch ( char * str ) {
    cout << str << endl;
  }

  return 0;
}


