#ifndef SENSOR_H_
#define SENSOR_H_

#include <vector>
#include <iostream>
#include "geom/vec_2f.h"
#include "util/mtrand.h"
#include "util/mwcrand.h"
#include "gui/brush.h"

extern MTRand mt;


class Sensor
{
	friend class Swarm;
	friend class CoverGrid;
	public:
		Sensor ( const vec_2f & startpos, const float mobility, const float radius, const unsigned int id );
      // start position, max. travel distance, and sensing radius

		void draw ( Brush & brush, const bool paths, float progress = 0.0 ) const;

    // genetic ops
		void scramble ();
		void scrambleStraight ();
		void mutate ( const float anglevar );


		inline const std::vector< vec_2f > & getRawPath () const
      { return m_path; }

    void setRawPath ( const std::vector< vec_2f > & newpath );

		std::vector< vec_2f > getPath () const;
      // get list of absolute coordinates this sensor visits

    float maxArea () const;
      // maximum raw area this sensor can cover

		inline float getRadius () const
      { return m_radius; }

    inline float getMobility () const
      { return m_mobility; }

    inline unsigned int getID () const
      { return m_id; }

	private:
		vec_2f m_startpos;
		float m_mobility;
		float m_radius;
		unsigned int m_id;

		std::vector< vec_2f > m_path; // movement plan / genome ( relative positions )
};

#endif /*SENSOR_H_*/
