#ifndef SIMULATION_H_
#define SIMULATION_H_

#include <iostream>
#include "gui/brush.h"
#include "util/mtrand.h"
#include "swarm.h"

extern MTRand mt;

class Simulation
{
	public:
		Simulation ( const float senseradius );
      // give typical sensor radius
		~Simulation ();


    struct drawopts_t
    {
      bool cover;
      bool sensors;
      bool paths;
    };

		void draw ( Brush & brush, const drawopts_t & opts ) const;

		void resetTime ();
		void step ( const float step );


		void addCluster ( const Sensor & base, const vec_2f & center, const float radius, const unsigned int count );
      // add a cluster of sensors with base's properties

		void addUniform ( const Sensor & base, const rect_f & area, const unsigned int count );

		void setTarget ( const circle & area, const bool target = true );

		std::vector< float > getTargetNCover () const;
			// get target n-cover

    std::vector< float > getMaxNCover () const;
      // get max theoretical target cover

		unsigned int N () const;
		void setN ( const unsigned int n );

		void scramble ();

		void optimizeN ();
			// one iteration per sensor

		void optimize ( unsigned int iterations );

		unsigned int totalIts () const;
			// how many iterations so far

		void clearSensors ();
		void clearTarget ();

	private:

		unsigned int m_covern;
		unsigned int m_its;
		float m_time;

		Swarm m_swarm;
};

#endif /*SIMULATION_H_*/
