
#include "swarm.h"

Swarm::Swarm ( const float senseradius )
	: m_cover( senseradius * 0.2, senseradius ), m_targetncover( 0 )
{}

Swarm::~Swarm () {}


bool Swarm::canInclude ( const Sensor & s ) const
{
	return true;
}


bool Swarm::canInclude ( const Swarm & s ) const
{
	return true;
}


void Swarm::merge ( const Swarm & source )
{
	for( unsigned int i = 0; i < source.m_sensors.size(); i++ )
		addSensor( source.m_sensors[ i ] );
}


void Swarm::addSensor ( const Sensor & s )
{
	m_sensors.push_back( s );
	applySensor( m_sensors.back() );
}


void Swarm::clear ()
{
	m_sensors.clear();
	m_cover.uncoverAll();
}


void Swarm::drawCover ( Brush & brush, const unsigned int cover ) const
{
  m_cover.drawCells( brush );
}


void Swarm::drawSensors ( Brush & brush, const bool paths, float progress ) const
{
	for( unsigned int i = 0; i < m_sensors.size(); i++ )
		m_sensors[ i ].draw( brush, paths, progress );
}


void Swarm::setTarget ( const circle & area, const bool target )
{
  m_cover.setTarget( area, target );
}


void Swarm::unTargetAll ()
{
  m_cover.unTargetAll();
}


void Swarm::setTargetNCover ( const unsigned int n )
  { m_targetncover = n; }


void Swarm::scramble ()
{
	for( unsigned int si = 0; si < m_sensors.size(); si++ )
		m_sensors[ si ].scramble();

	m_cover.uncoverAll();
	for( unsigned int si = 0; si < m_sensors.size(); si++ )
		applySensor( m_sensors[ si ] );
}


void Swarm::mutateOneRand ()
{
	if( m_sensors.empty() )
		return;
	tryMutate( m_sensors[ mwc.randInt() % m_sensors.size() ] );
}


void Swarm::mutateAllOnce ()
{
	if( m_sensors.empty() )
		return;
	std::vector< unsigned int > sensoris;
	sensoris.resize( m_sensors.size(), 0 );
	for( unsigned int i = 0; i < sensoris.size(); i++ )
		sensoris[ i ] = i;
	for( unsigned int i = 0; i < sensoris.size() - 1; i++ )
	{
		const unsigned int j = mwc.randInt() % ( sensoris.size() - i ) + i;
		if( i != j )
		{
			const unsigned int temp = sensoris[ i ];
			sensoris[ i ] = sensoris[ j ];
			sensoris[ j ] = temp;
		}
	}

	for( unsigned int i = 0; i < sensoris.size(); i++ )
		tryMutate( m_sensors[ sensoris[ i ] ] );
}


unsigned int Swarm::sensorCount () const
  {	return m_sensors.size(); }


float Swarm::maxArea () const
{
  float area = 0;
  for( unsigned int si = 0; si < m_sensors.size(); si++ )
		area += m_sensors[ si ].maxArea();
  return area;
}


void Swarm::applySensor ( const Sensor & s )
{
  std::vector< vec_2f > path = s.getPath();
  for( unsigned int i = 0; i < path.size(); i++ )
    m_cover.cover( circle( path[ i ], s.getRadius() ), s.getID() );
}


void Swarm::unapplySensor ( const Sensor & s )
{
  std::vector< vec_2f > path = s.getPath();
  for( unsigned int i = 0; i < path.size(); i++ )
    m_cover.uncover( circle( path[ i ], s.getRadius() ), s.getID() );
}


bool Swarm::tryMutate ( Sensor & s )
{
  const float anglevar = TWOPI * 0.2;
  const float straightchance = 0.05;

	CoverScore oldscore = m_cover.getScore();
	std::vector< vec_2f > oldpath = s.getRawPath();

	unapplySensor( s );
	if( mt.rand() < straightchance )
    s.scrambleStraight();
  else
    s.mutate( anglevar );
	applySensor( s );

	if( m_cover.getScore() >= oldscore )
		return true;

	unapplySensor( s );
	s.setRawPath( oldpath );
	applySensor( s );
	return false;
}
