#ifndef SWARM_H_
#define SWARM_H_

#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>
#include <vector>
#include "gui/brush.h"
#include "geom/vec_2f.h"
#include "util/mwcrand.h"
#include "util/powertable.h"
#include "cover/cover.h"
#include "sensor.h"

extern MWCRand mwc;


class Swarm
{
	public:
		Swarm ( const float senseradius );
      // give typical sensor radius to expect
		~Swarm ();

		//////////////////////////////////////

		bool canInclude ( const Sensor & s ) const;
		bool canInclude ( const Swarm & s ) const;
			// return true if s can communicate with any sensor in this swarm

		void merge ( const Swarm & source );
			// merge the source swarm into this one

		void addSensor ( const Sensor & s );

		void clear ();

		//////////////////////////////////////

		void drawCover ( Brush & brush, const unsigned int cover ) const;

		void drawSensors ( Brush & brush, const bool paths, float progress = 0.0 ) const;

		void setTarget ( const circle & area, const bool target );
			// mark the given area as target, or not
    void unTargetAll ();

		void setTargetNCover ( const unsigned int n );
			// set maximum ncoverage to optimize, or 0 for unlimited

		void scramble ();
			// randomize all sensor movement plans

		void mutateOneRand ();
			// try one random change to one random movement plan

		void mutateAllOnce ();
			// try one mutation on each sensor, in random order

		unsigned int sensorCount () const;

		float maxArea () const;
      // maximum raw area this swarm can cover

		inline const Cover & getCover () const
      { return m_cover; }


	private:

    void applySensor ( const Sensor & s );
    void unapplySensor ( const Sensor & s );
      // add/remove the sensor to/from the cover

		bool tryMutate ( Sensor & s );
			// try a mutation on sensor s, undo it if it fails
			// return true if it succeeds


		Cover m_cover;
		unsigned int m_targetncover; // maximum n-coverage of target
		std::vector< Sensor > m_sensors;
};

#endif /*SWARM_H_*/
