#ifndef MWCRAND_H_
#define MWCRAND_H_

// multiply-with-carry RNG
// http://groups.google.com/group/sci.stat.math/browse_thread/thread/b2cdf0a0a96c7375/fa034083b193adf0?&hl=en#fa034083b193adf0

// some values of a such that a*2^32-1 and a*2^31-1 are prime
//	1791398085 1929682203 1683268614 1965537969 1675393560 
//	1967773755 1517746329 1447497129 1655692410 1606218150 
//	2051013963 1075433238 1557985959 1781943330 1893513180 
//	1631296680 2131995753 2083801278 1873196400 1554115554

class MWCRand
{
	public:
		MWCRand () : a(1554115554), x(1823812398), c(1234567),
			bitwad(0), bitsleft(0)
		{}

		inline void seed ( unsigned long long seed )
		{ x = seed; }

		inline unsigned long randInt ()
		{
			unsigned long long axc = a * x + c;
			c = axc >> 32;
			x = axc & 0xffffffff;
			return (unsigned long)x;
		}
		
		inline float rand ()
		{
			return (float)randInt() / (float)( 0xffffffff );
		}
		
		inline unsigned long randBits ( unsigned int bits )
		{
			if( bits > 32 )
				bits = 32;
			if( bits > bitsleft ){
				bitwad = randInt();
				bitsleft = 32;
			}
			unsigned long val = 0xffffffff;
		  val = ~(val << bits);
		  val = val & bitwad;
		  bitwad = bitwad >> bits;
		  bitsleft -= bits;
		  return val;
		}

	private:
		const unsigned long long a;
		unsigned long long x;
		unsigned long long c;
		unsigned long bitwad;
		unsigned int bitsleft;
};

#endif /*MWCRAND_H_*/
