#ifndef QUICKSET_H_
#define QUICKSET_H_

// this class is a kind of estimated set of unique ids

#define QUICKSET_BANKS		4		// quickset can manage ( banks * 32 ) unique values

class QuickSet
{
	public:
		QuickSet () { clear(); }

		inline bool insert ( const unsigned int id )
		{
			m_accumtotal++;
			unsigned int & bank = m_memberbank[ ( id % ( QUICKSET_BANKS * 32 ) ) / 32 ];
			const unsigned int mask = (unsigned int)1 << ( id % 32 );
			if( ( bank & mask ) == 0 )
			{
				m_members++;
				bank |= mask;
				return true;
			}
			else
				return false;
		}


		inline void insertAll ()
		{
			for( unsigned int i = 0; i < QUICKSET_BANKS; i++ )
				m_memberbank[ i ] = ~(unsigned int)0;
			m_members = QUICKSET_BANKS * 32;
			m_accumtotal += QUICKSET_BANKS * 32;
		}


		inline bool remove ( const unsigned int id )
		{
			m_accumtotal--;
			unsigned int & bank = m_memberbank[ ( id % ( QUICKSET_BANKS * 32 ) ) / 32 ];
			const unsigned int mask = (unsigned int)1 << ( id % 32 );
			if( ( bank & mask ) != 0 )
			{
				if( m_members > 0 )
					m_members--;
				bank &= ~mask;
				return true;
			}
			else
				return false;
		}


		inline void clear ()
		{
			for( unsigned int i = 0; i < QUICKSET_BANKS; i++ )
				m_memberbank[ i ] = 0;
			m_members = 0;
			m_accumtotal = 0;
		}


		inline bool isMember ( const unsigned int id ) const
		{
			const unsigned int & bank = m_memberbank[ ( id % ( QUICKSET_BANKS * 32 ) ) / 32 ];
			return bank & ( (unsigned int)1 << ( id % 32 ) ) != 0;
		}


		inline unsigned int collisions ( const QuickSet & rhs ) const
			// number of members these two have in common
		{
			unsigned int cols = 0;
			for( unsigned int bi = 0; bi < QUICKSET_BANKS; bi++ )
			{
				unsigned int b = m_memberbank[ bi ] & rhs.m_memberbank[ bi ];
				unsigned int n = 0;
				while( b )
				{
					b &= ( b - 1 );
					n++;
				}
				cols += n;
			}
			return cols;
		}


		inline unsigned int members () const
			{ return m_members; }

		inline int accumTotal () const // inserts - removes, regardless of success
			{ return m_accumtotal; }

	private:
		unsigned int m_memberbank[ QUICKSET_BANKS ];
		unsigned int m_members;
		int m_accumtotal;
};



#endif /*QUICKSET_H_*/
