
#include "tallyvector.h"


void TallyVector::print () const
{
  for( unsigned int i = 0; i < m_tallies.size(); i++ )
  {
    cout << m_tallies[ i ];
    if( i + 1 < m_tallies.size() )
      cout << ", ";
    else
      cout << endl;
  }
  cout.flush();
}


void TallyVector::incr ( const unsigned int i )
{
  if( i + 1 > m_tallies.size() )
    m_tallies.resize( i + 1, 0 );
  m_tallies[ i ]++;
}


void TallyVector::decr ( const unsigned int i )
{
  if( i + 1 > m_tallies.size() )
    throw( "tallyvector: decr: invalid i" );
  if( m_tallies[ i ] > 0 )
    m_tallies[ i ]--;
  // decrementing zero can't be an error here, because some sensor
  //  ids may collide in quickset
}


void TallyVector::set ( const unsigned int i, const unsigned int tally )
{
	if( i + 1 > m_tallies.size() )
		m_tallies.resize( i + 1, 0 );
	m_tallies[ i ] = tally;
}


void TallyVector::clear ()
{
  m_tallies.clear();
}


const TallyVector & TallyVector::operator+= ( const TallyVector & rhs )
{
  if( rhs.m_tallies.size() > m_tallies.size() )
    m_tallies.resize( rhs.m_tallies.size(), 0 );
  for( unsigned int i = 0; i < rhs.m_tallies.size(); i++ )
    m_tallies[ i ] += rhs.m_tallies[ i ];
  return *this;
}


const TallyVector & TallyVector::operator-= ( const TallyVector & rhs )
{
  for( unsigned int i = 0; i < rhs.m_tallies.size(); i++ )
    if( i + 1 > m_tallies.size() )
    {
      if( rhs.m_tallies[ i ] > 0 )
        throw( "tallyvector: operator-=: zero - ?" );
    }
    else
    {
      if( rhs.m_tallies[ i ] > m_tallies[ i ] )
        throw( "tallyvector: operator-=: ? - ? < 0" );
      m_tallies[ i ] -= rhs.m_tallies[ i ];
    }
  return *this;
}
